package aspects;

import java.security.AccessControlException;
import java.security.AccessController;
import java.io.File;

/**
 * @author Angel Nunez (angel.nunez@gmail.com)
 */
public aspect FileDeletionRestriction {

	pointcut untrustedMethods(): execution(* untrusted.*.*(..));
	pointcut trustedMethods(): execution(* framework.*.*(..));
	pointcut resourceAccess(): call(* File.delete(..));
	
	AC_Automaton automaton = new AC_Automaton();
	
	before() : resourceAccess() {
		if(!automaton.inLegalState()) {
			throw new AccessControlException("No permissions to access file.");
		}
	}
	
	Object around() : untrustedMethods() {
		automaton.enterUntrusted();
		try{
			return proceed();
		}
		finally {
			automaton.exitUntrusted();
		}
	}
	
	Object around() : trustedMethods() {
		automaton.enterTrusted();
		try {
			return proceed();
		}
		finally {
			automaton.exitTrusted();
		}
	}
	
	Object around() : call(* AccessController.doPrivileged(..)) {
		automaton.enterDoPriv();
		try {
			return proceed();
		}
		finally {
			automaton.exitDoPriv();
		}
	}
}
